==> dnsdbq_2.3.0.orig.tar.gz: dnsdbq-2.3.0/ns_ttl.c <==
/*
 * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
 * Copyright (c) 1996,1999 by Internet Software Consortium.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* ISC "Id: ns_ttl.c,v 8.10 2005/03/31 02:02:29 marka Exp $" */

/* Import. */

#include <sys/types.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include "ns_ttl.h"

/* Forward. */

static int fmt1(u_long t, char s, char **buf, size_t *buflen);

/* Macros. */

#define T(x) {if ((x) < 0) return (-1); else (void)NULL;}
#define A(c) {if (dstlen < 2) return (-1); else *dst++ = c, *dst = '\0';}

/* Public. */

int
ns_format_ttl(u_long src, char *dst, size_t dstlen) {
	u_long secs, mins, hours, days, years;
	char *odst = dst;

	secs = src % 60;   src /= 60;
	mins = src % 60;   src /= 60;
	hours = src % 24;  src /= 24;
	days = src % 365;  src /= 365;
	years = src;       src = 0;

	if (years != 0) {
		A('~');
		T(fmt1(years, 'y', &dst, &dstlen));
	}
	if (days != 0) {
		if (dst != odst)
			A('\040');
		A('~');
		T(fmt1(days, 'd', &dst, &dstlen));
	}
	if (years == 0 && hours != 0) {
		if (dst != odst)
			A('\040');
		T(fmt1(hours, 'h', &dst, &dstlen));
	}
	if (years == 0 && mins != 0) {
		if (dst != odst)
			A('\040');
		T(fmt1(mins, 'm', &dst, &dstlen));
	}
	if ((years == 0 && days == 0 && secs != 0) ||
	    (years == 0 && days == 0 && hours == 0 && mins == 0))
	{
		if (dst != odst)
			A('\040');
		T(fmt1(secs, 's', &dst, &dstlen));
	}

	return (int)(dst - odst);
}

int
ns_parse_ttl(const char *src, u_long *dst) {
	u_long ttl, tmp;
	int ch, digits, dirty;

	ttl = 0;
	tmp = 0;
	digits = 0;
	dirty = 0;
	while ((ch = *src++) != '\0') {
		if (!isascii(ch) || !isprint(ch))
			goto einval;
		if (isdigit(ch)) {
			tmp *= 10;
			tmp += (unsigned)(ch - '0');
			digits++;
			continue;
		}
		if (digits == 0)
			goto einval;
		if (islower(ch))
			ch = toupper(ch);
		switch (ch) {
		case 'W':  tmp *= 7;
			/* FALLTHROUGH */
		case 'D':  tmp *= 24;
			/* FALLTHROUGH */
		case 'H':  tmp *= 60;
			/* FALLTHROUGH */
		case 'M':  tmp *= 60;
			/* FALLTHROUGH */
		case 'S':  break;
		default:   goto einval;
		}
		ttl += tmp;
		tmp = 0;
		digits = 0;
		dirty = 1;
	}
	if (digits > 0) {
		if (dirty)
			goto einval;
		else
			ttl += tmp;
	} else if (!dirty)
		goto einval;
	*dst = ttl;
	return (0);

 einval:
	errno = EINVAL;
	return (-1);
}

/* Private. */

static int
fmt1(u_long t, char s, char **buf, size_t *buflen) {
	char tmp[50];
	size_t len;

	len = (size_t) sprintf(tmp, "%lu%c", t, s);
	if (len + 1 > *buflen)
		return (-1);
	strcpy(*buf, tmp);
	*buf += len;
	*buflen -= len;
	return (0);
}

==> dnsdbq_2.6.8.orig.tar.gz: dnsdbq-2.6.8/ns_ttl.c <==
/*
 * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
 * Copyright (c) 1996,1999 by Internet Software Consortium.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* ISC "Id: ns_ttl.c,v 8.10 2005/03/31 02:02:29 marka Exp $" */

/* Import. */

#include <sys/types.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include "ns_ttl.h"

/* Forward. */

static int fmt1(u_long t, char s, char **buf, size_t *buflen);

/* Macros. */

#define T(x) {if ((x) < 0) return -1; else (void)NULL;}
#define A(c) {if (dstlen < 2) return -1; else *dst++ = c, *dst = '\0';}

/* Public. */

int
ns_format_ttl(u_long src, char *dst, size_t dstlen) {
	u_long secs, mins, hours, days, years;
	char *odst = dst;

	secs = src % 60;   src /= 60;
	mins = src % 60;   src /= 60;
	hours = src % 24;  src /= 24;
	days = src % 365;  src /= 365;
	years = src;       src = 0;

	if (years != 0) {
		A('~');
		T(fmt1(years, 'y', &dst, &dstlen));
	}
	if (days != 0) {
		if (dst != odst)
			A('\040');
		A('~');
		T(fmt1(days, 'd', &dst, &dstlen));
	}
	if (years == 0 && hours != 0) {
		if (dst != odst)
			A('\040');
		T(fmt1(hours, 'h', &dst, &dstlen));
	}
	if (years == 0 && mins != 0) {
		if (dst != odst)
			A('\040');
		T(fmt1(mins, 'm', &dst, &dstlen));
	}
	if ((years == 0 && days == 0 && secs != 0) ||
	    (years == 0 && days == 0 && hours == 0 && mins == 0))
	{
		if (dst != odst)
			A('\040');
		T(fmt1(secs, 's', &dst, &dstlen));
	}

	return (int)(dst - odst);
}

int
ns_parse_ttl(const char *src, u_long *dst) {
	u_long ttl, tmp;
	int ch, digits, dirty;

	ttl = 0;
	tmp = 0;
	digits = 0;
	dirty = 0;
	while ((ch = *src++) != '\0') {
		if (!isascii(ch) || !isprint(ch))
			goto einval;
		if (isdigit(ch)) {
			tmp *= 10;
			tmp += (unsigned)(ch - '0');
			digits++;
			continue;
		}
		if (digits == 0)
			goto einval;
		if (islower(ch))
			ch = toupper(ch);
		switch (ch) {
		case 'W':  tmp *= 7;
			/* FALLTHROUGH */
		case 'D':  tmp *= 24;
			/* FALLTHROUGH */
		case 'H':  tmp *= 60;
			/* FALLTHROUGH */
		case 'M':  tmp *= 60;
			/* FALLTHROUGH */
		case 'S':  break;
		default:   goto einval;
		}
		ttl += tmp;
		tmp = 0;
		digits = 0;
		dirty = 1;
	}
	if (digits > 0) {
		if (dirty)
			goto einval;
		else
			ttl += tmp;
	} else if (!dirty)
		goto einval;
	*dst = ttl;
	return 0;

 einval:
	errno = EINVAL;
	return -1;
}

/* Private. */

static int
fmt1(u_long t, char s, char **buf, size_t *buflen) {
	char tmp[50];
	size_t len;

	len = (size_t) sprintf(tmp, "%lu%c", t, s);
	if (len + 1 > *buflen)
		return -1;
	strcpy(*buf, tmp);
	*buf += len;
	*buflen -= len;
	return 0;
}
